/*
libutil -- stuff dealing with articles

Written by Cornelius Krasel <krasel@wpxx02.toxi.uni-wuerzburg.de>.
Copyright 1998, 1999.

See file COPYING for restrictions on the use of this software.
*/

#include "leafnode.h"
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <sys/stat.h>
#include <unistd.h>
#include "mastring.h"

/*
 * find a header in an article and return it
 */
char *
fgetheader(FILE * f, const char *header)
{
    char *hdr, *p;
    int hlen, next;

    if (!f)
	return NULL;
    rewind(f);
    debug = 0;
    next = 0;
    hdr = NULL;
    p = NULL;
    hlen = (int)strlen(header);
    while ((p = getaline(f)) && *p) {	/* read only headers */
	/* allow for multiline headers */
	if (isspace((unsigned char)*p) && hdr != NULL && next) {
	    hdr = critrealloc(hdr, strlen(hdr) + strlen(p) + 2,
			      "Allocating space for multiline headers");
	    strcat(hdr, "\n");	/* Flawfinder: ignore */
	    strcat(hdr, p);	/* Flawfinder: ignore */
	} else if ((strncasecmp(p, header, hlen) == 0)) {
	    p += hlen;
	    if (*p == ':')
		p++;
	    while (p && *p && isspace((unsigned char)*p))
		p++;
	    hdr = strdup(p);
	    next = 1;
	} else
	    next = 0;
    }
    debug = debugmode;
    rewind(f);
    return hdr;
}

char *
getheader(const char *filename, const char *header)
{
    FILE *f;
    char *hdr;
    struct stat st;

    if (stat(filename, &st) || !S_ISREG(st.st_mode))
	return NULL;
    if ((f = fopen(filename, "r")) == NULL)
	return NULL;
    hdr = fgetheader(f, header);
    fclose(f);
    return hdr;
}

/*
 * store articles in newsgroups which are already stored in
 * $SPOOLDIR/message.id/
 */
void
store(const char *filename,
      FILE * filehandle, const char *newsgroups, const char *msgid)
{
    char tmp[50];
    static struct newsgroup *cg;
    mastr *xrefincase = mastr_new(4096);
    const char *p;
    char *q;

    if (verbose > 2)
	printf("storing %s: %s\n", msgid, newsgroups);

    p = newsgroups;
    while (p && *p) {
	q = strchr(p, ',');
	if (q)
	    *q++ = '\0';
	if (*p) {
	    if (!cg || strcmp(cg->name, p)) {
		cg = findgroup(p);
		if (cg) {
		    if (isinteresting(cg->name) || create_all_links)
			(void)chdirgroup(p, TRUE);
		    else
			cg = NULL;
		}
	    }
	    if (cg) {
		do {
		    sprintf(tmp, "%lu", ++cg->last);
		    errno = 0;
		    if (verbose > 2)
			printf("..as article %lu in %s\n", cg->last, cg->name);
		} while (link(filename, tmp) < 0 && errno == EEXIST);
		if (errno)
		    syslog(LOG_ERR, "error linking %s into %s: %m",
			   filename, p);
		else {
		    sprintf(tmp, ":%lu", cg->last);
		    mastr_vcat(xrefincase, " ", cg->name, tmp, NULL);
		}
	    } else {
		if (verbose > 2)
		    printf(".. discarding unknown group %s\n", p);
	    }
	}
	p = q;
    }
    fprintf(filehandle, "Xref: %s%s\n", fqdn, mastr_str(xrefincase));
    mastr_delete(xrefincase);
}
